@@ -32,8 +32,6 @@
 #include <glade/glade-build.h>
 #include <libgnomeui/libgnomeui.h>
 
-#include <libgnomeui/libgnomeuiP.h>
-
 #include <libbonoboui.h>
 
 #define INT(s)   (strtol ((s), NULL, 0))
@@ -307,7 +305,7 @@ dialog_build_children(GladeXML *self, GtkWidget *w,
 
     glade_standard_build_children (self, w, info);
 
-    aa = dialog->_priv->action_area;
+    aa = dialog->action_area;
     if (!aa)
 	return;
 
@@ -474,13 +472,13 @@ druid_page_edge_new (GladeXML *xml, GType widget_type,
 	    title = value;
 	else if (!strcmp (name, "logo_image")) {
 	    if (logo)
-		gdk_pixbuf_unref (logo);
+		g_object_unref (G_OBJECT (logo));
 	    filename = glade_xml_relative_file (xml, value);
 	    logo = gdk_pixbuf_new_from_file (filename, NULL);
 	    g_free (filename);
 	} else if (!strcmp (name, "watermark_image")) {
 	    if (watermark)
-		gdk_pixbuf_unref (watermark);
+		g_object_unref (G_OBJECT (watermark));
 	    filename = glade_xml_relative_file (xml, value);
 	    watermark = gdk_pixbuf_new_from_file (filename, NULL);
 	    g_free (filename);
@@ -495,11 +493,11 @@ druid_page_edge_new (GladeXML *xml, GType widget_type,
     
 
     if (logo)
-	gdk_pixbuf_unref (logo);
+	g_object_unref (G_OBJECT (logo));
     if (watermark)
-	gdk_pixbuf_unref (watermark);
+	g_object_unref (G_OBJECT (watermark));
     if (top_watermark)
-	gdk_pixbuf_unref (top_watermark);
+	g_object_unref (G_OBJECT (top_watermark));
 
     return druid;
 }
@@ -591,7 +589,7 @@ dialog_find_internal_child(GladeXML *xml, GtkWidget *parent,
     if (!strcmp(childname, "vbox"))
 	return GNOME_DIALOG(parent)->vbox;
     else if (!strcmp (childname, "action_area"))
-	return GNOME_DIALOG (parent)->_priv->action_area;
+	return GNOME_DIALOG (parent)->action_area;
 
     return NULL;
 }
@@ -623,7 +621,7 @@ propertybox_find_internal_child(GladeXML *xml, GtkWidget *parent,
     if (!strcmp(childname, "vbox"))
 	return GNOME_DIALOG(parent)->vbox;
     else if (!strcmp(childname, "action_area"))
-	return GNOME_DIALOG(parent)->_priv->action_area;
+	return GNOME_DIALOG(parent)->action_area;
     else if (!strcmp(childname, "notebook"))
 	return GNOME_PROPERTY_BOX(parent)->notebook;
     else if (!strcmp(childname, "ok_button"))
